@@ -7,6 +7,9 @@ module Cabal
 #ifdef ENABLE_CABAL
 import Stack
 import Control.Exception (IOException, catch)
+import Control.Monad (when)
+import Control.Monad.Trans.Class (lift)
+import Control.Monad.Trans.State (execStateT, modify)
 import Data.Char (isSpace)
 import Data.List (foldl', nub, sort, find, isPrefixOf, isSuffixOf)
 #if __GLASGOW_HASKELL__ < 709
@@ -24,12 +27,13 @@ import Distribution.Simple.LocalBuildInfo (LocalBuildInfo(..), ComponentLocalBui
 #endif
     componentBuildInfo, foldComponent)
 import Distribution.Simple.Compiler (PackageDB(..))
+import Distribution.Simple.Command (CommandParse(..), commandParseArgs)
 import Distribution.Simple.GHC (componentGhcOptions)
 import Distribution.Simple.Program (defaultProgramConfiguration)
 import Distribution.Simple.Program.Db (lookupProgram)
 import Distribution.Simple.Program.Types (ConfiguredProgram(programVersion), simpleProgram)
 import Distribution.Simple.Program.GHC (GhcOptions(..), renderGhcOptions)
-import Distribution.Simple.Setup (ConfigFlags(..), defaultConfigFlags, toFlag)
+import Distribution.Simple.Setup (ConfigFlags(..), defaultConfigFlags, configureCommand, toFlag)
 #if __GLASGOW_HASKELL__ >= 709
 import Distribution.Utils.NubList
 import qualified Distribution.Simple.GHC as GHC(configure)
@@ -38,7 +42,7 @@ import Distribution.Verbosity (silent)
 import Distribution.Version (Version(..))
 
 import System.IO.Error (ioeGetErrorString)
-import System.Directory (doesFileExist, getDirectoryContents)
+import System.Directory (doesFileExist, doesDirectoryExist, getDirectoryContents)
 import System.FilePath (takeDirectory, splitFileName, (</>))
 
 
@@ -110,30 +114,44 @@ stackifyFlags cfg (Just si) = cfg { configDistPref    = toFlag dist
 -- via: https://groups.google.com/d/msg/haskell-stack/8HJ6DHAinU0/J68U6AXTsasJ
 -- cabal configure --package-db=clear --package-db=global --package-db=$(stack path --snapshot-pkg-db) --package-db=$(stack path --local-pkg-db)
 
-getPackageGhcOpts :: FilePath -> Maybe StackConfig -> IO (Either String [String])
-getPackageGhcOpts path mbStack = do
+getPackageGhcOpts :: FilePath -> Maybe StackConfig -> [String] -> IO (Either String [String])
+getPackageGhcOpts path mbStack opts = do
     getPackageGhcOpts' `catch` (\e -> do
         return $ Left $ "Cabal error: " ++ (ioeGetErrorString (e :: IOException)))
   where
     getPackageGhcOpts' :: IO (Either String [String])
     getPackageGhcOpts' = do
         genPkgDescr <- readPackageDescription silent path
-        let cfgFlags'' = (defaultConfigFlags defaultProgramConfiguration)
-                            { configDistPref = toFlag $ takeDirectory path </> "dist"
-                            -- TODO: figure out how to find out this flag
-                            , configUserInstall = toFlag True
-                            }
-        let cfgFlags'  = stackifyFlags cfgFlags'' mbStack
-        let sandboxConfig = takeDirectory path </> "cabal.sandbox.config"
-        exists <- doesFileExist sandboxConfig
+        distDir     <- getDistDir
+
+        let programCfg = defaultProgramConfiguration
+        let initCfgFlags = (defaultConfigFlags programCfg)
+                             { configDistPref = toFlag distDir
+                             -- TODO: figure out how to find out this flag
+                             , configUserInstall = toFlag True
+
+                             -- configure with --enable-tests to include test dependencies/modules
+                             , configTests = toFlag True
+
+                             -- configure with --enable-benchmarks to include benchmark dependencies/modules
+                             , configBenchmarks = toFlag True
+                             }
+        let initCfgFlags' = stackifyFlags initCfgFlags mbStack
+
+        cfgFlags <- flip execStateT initCfgFlags' $ do
+          let sandboxConfig = takeDirectory path </> "cabal.sandbox.config"
+
+          exists <- lift $ doesFileExist sandboxConfig
+          when (exists) $ do
+            sandboxPackageDb <- lift $ getSandboxPackageDB sandboxConfig
+            modify $ \x -> x { configPackageDBs = [Just sandboxPackageDb] }
+
+          let cmdUI = configureCommand programCfg
+          case commandParseArgs cmdUI True opts of
+            CommandReadyToGo (modFlags, _) -> modify modFlags
+            CommandErrors (e:_) -> error e
+            _ -> return ()
 
-        cfgFlags <- case exists of
-                         False -> return cfgFlags'
-                         True -> do
-                             sandboxPackageDb <- getSandboxPackageDB sandboxConfig
-                             return $ cfgFlags'
-                                          { configPackageDBs = [Just sandboxPackageDb]
-                                          }
         localBuildInfo <- configure (genPkgDescr, emptyHookedBuildInfo) cfgFlags
         let pkgDescr = localPkgDescr localBuildInfo
         let baseDir = fst . splitFileName $ path
@@ -166,6 +184,14 @@ getPackageGhcOpts path mbStack = do
                 return $ Right $ renderGhcOptions ghcVersion ghcOpts
 #endif
 
+    -- returns the right 'dist' directory in the case of a sandbox
+    getDistDir = do
+        let dir = takeDirectory path </> "dist"
+        exists <- doesDirectoryExist dir
+        if not exists then return dir else do
+            contents <- getDirectoryContents dir
+            return . maybe dir (dir </>) $ find ("dist-sandbox-" `isPrefixOf`) contents
+
 pkgLibName :: PackageDescription -> Maybe PackageName
 pkgLibName pkgDescr = if hasLibrary pkgDescr
                       then Just $ pkgName . package $ pkgDescr
@@ -217,8 +243,8 @@ findCabalFile dir = do
 
 # else
 
-getPackageGhcOpts :: FilePath -> IO (Either String [String])
-getPackageGhcOpts _ = return $ Right []
+getPackageGhcOpts :: FilePath -> [String] -> IO (Either String [String])
+getPackageGhcOpts _ _ = return $ Right []
 
 findCabalFile :: FilePath -> IO (Maybe FilePath)
 findCabalFile _ = return Nothing
