@@ -53,17 +53,21 @@ data HDevTools
     | Check
         { socket  :: Maybe FilePath
         , ghcOpts :: [String]
+        , cabalOpts :: [String]
         , path    :: Maybe String
         , file    :: String
+        , json    :: Bool
         }
     | ModuleFile
         { socket  :: Maybe FilePath
         , ghcOpts :: [String]
+        , cabalOpts :: [String]
         , module_ :: String
         }
     | Info
         { socket     :: Maybe FilePath
         , ghcOpts    :: [String]
+        , cabalOpts  :: [String]
         , path       :: Maybe String
         , file       :: String
         , identifier :: String
@@ -71,6 +75,7 @@ data HDevTools
     | Type
         { socket  :: Maybe FilePath
         , ghcOpts :: [String]
+        , cabalOpts  :: [String]
         , path    :: Maybe String
         , file    :: String
         , line    :: Int
@@ -79,6 +84,7 @@ data HDevTools
     | FindSymbol
         { socket :: Maybe FilePath
         , ghcOpts :: [String]
+        , cabalOpts :: [String]
         , symbol :: String
         , files :: [String]
         }
@@ -86,25 +92,28 @@ data HDevTools
 
 dummyAdmin :: HDevTools
 dummyAdmin = Admin
-    { socket = Nothing
+    { socket       = Nothing
     , start_server = False
-    , noDaemon = False
-    , status = False
-    , stop_server = False
+    , noDaemon     = False
+    , status       = False
+    , stop_server  = False
     }
 
 dummyCheck :: HDevTools
 dummyCheck = Check
     { socket  = Nothing
     , ghcOpts = []
+    , cabalOpts = []
     , path    = Nothing
     , file    = ""
+    , json    = False
     }
 
 dummyModuleFile :: HDevTools
 dummyModuleFile = ModuleFile
     { socket  = Nothing
     , ghcOpts = []
+    , cabalOpts = []
     , module_ = ""
     }
 
@@ -112,6 +121,7 @@ dummyInfo :: HDevTools
 dummyInfo = Info
     { socket     = Nothing
     , ghcOpts    = []
+    , cabalOpts = []
     , path       = Nothing
     , file       = ""
     , identifier = ""
@@ -121,6 +131,7 @@ dummyType :: HDevTools
 dummyType = Type
     { socket  = Nothing
     , ghcOpts = []
+    , cabalOpts = []
     , path    = Nothing
     , file    = ""
     , line    = 0
@@ -131,6 +142,7 @@ dummyFindSymbol :: HDevTools
 dummyFindSymbol = FindSymbol
     { socket = Nothing
     , ghcOpts = []
+    , cabalOpts = []
     , symbol = ""
     , files = []
     }
@@ -148,14 +160,25 @@ check :: Annotate Ann
 check = record dummyCheck
     [ socket   := def += typFile      += help "socket file to use"
     , ghcOpts  := def += typ "OPTION" += help "ghc options"
+#ifdef ENABLE_CABAL
+    , cabalOpts := def += typ "OPTION"  += help "cabal options"
+#else
+    , cabalOpts := def += ignore
+#endif
     , path     := def += typFile      += help "path to target file"
     , file     := def += typFile      += argPos 0 += opt ""
+    , json     := def                 += help "render output as JSON"
     ] += help "Check a haskell source file for errors and warnings"
 
 moduleFile :: Annotate Ann
 moduleFile = record dummyModuleFile
     [ socket   := def += typFile += help "socket file to use"
     , ghcOpts  := def += typ "OPTION" += help "ghc options"
+#ifdef ENABLE_CABAL
+    , cabalOpts := def += typ "OPTION"  += help "cabal options"
+#else
+    , cabalOpts := def += ignore
+#endif
     , module_  := def += typ "MODULE" += argPos 0
     ] += help "Get the haskell source file corresponding to a module name"
 
@@ -163,6 +186,11 @@ info :: Annotate Ann
 info = record dummyInfo
     [ socket     := def += typFile      += help "socket file to use"
     , ghcOpts    := def += typ "OPTION" += help "ghc options"
+#ifdef ENABLE_CABAL
+    , cabalOpts := def += typ "OPTION"  += help "cabal options"
+#else
+    , cabalOpts := def += ignore
+#endif
     , path       := def += typFile      += help "path to target file"
     , file       := def += typFile      += argPos 0 += opt ""
     , identifier := def += typ "IDENTIFIER" += argPos 1
@@ -172,6 +200,11 @@ type_ :: Annotate Ann
 type_ = record dummyType
     [ socket   := def += typFile += help "socket file to use"
     , ghcOpts  := def += typ "OPTION" += help "ghc options"
+#ifdef ENABLE_CABAL
+    , cabalOpts := def += typ "OPTION"  += help "cabal options"
+#else
+    , cabalOpts := def += ignore
+#endif
     , path     := def += typFile      += help "path to target file"
     , file     := def += typFile      += argPos 0 += opt ""
     , line     := def += typ "LINE"   += argPos 1
@@ -182,6 +215,11 @@ findSymbol :: Annotate Ann
 findSymbol = record dummyFindSymbol
     [ socket   := def += typFile += help "socket file to use"
     , ghcOpts  := def += typ "OPTION" += help "ghc options"
+#ifdef ENABLE_CABAL
+    , cabalOpts := def += typ "OPTION"  += help "cabal options"
+#else
+    , cabalOpts := def += ignore
+#endif
     , symbol   := def += typ "SYMBOL" += argPos 0
     , files    := def += typFile += args
     ] += help "List the modules where the given symbol could be found"
